<?php  // Make EMACS Happy: -*- Mode: PHP; c-basic-offset: 4; tab-width: 4 -*-
// {{{ Header

// SHA-1 implementation
// $Id: sha1.inc.php,v 1.5 2005/01/09 14:45:23 mina86 Exp $
// Copyright (c) 2003-2005 by Michal Nazarewicz (mina86/at/tlen.pl)
//
// This software is OSI Certified Open Source Software.
// OSI Certified is a certification mark of the Open Source Initiative.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// }}}
// {{{ Security

defined('IN_PHP_SCRIPT') or die('Hacking attempt');

// }}}

//
// You have to define IN_PHP_SCRIPT before including this file:
//
//   defined('IN_PHP_SCRIPT') or define('IN_PHP_SCRIPT', true);
//
// After including this file you will be able to use the fallowing
// functions:
//    sha1($str)
//    sha1_raw($str)
//    sha1_base64($str [, $pad = false])
//

// {{{ Function already exist?

if (function_exists('sha1')) {
	$sha1 = 'built in sha1() function';
	if ((int)PHP_VERSION>=5) {
		function &sha1_raw($str) {
			return sha1($str, true);
		}
	} else {
		function &sha1_raw($str) {
			return pack("H*", sha1($str));
		}
	}

// }}}
// {{{ So maybe we have mhash?

} elseif (function_exists('mhash')) {
	$sha1 = 'MHash library';

	function &sha1($str) {
		return bin2hex(mhash(MHASH_SHA1, $str));
	}

	function &sha1_raw($str) {
		return mhash(MHASH_SHA1, $str);
	}

// }}}
// {{{ So we must use own implementation

} else {
	$sha1 = 'own implementation';

	function _sha1_s($X, $n = 1) {
		return ($X << $n) | (($X & 0x80000000)?
			 (($X>>1) & 0x07fffffff | 0x40000000)>>(31-$n):$X>>(32-$n));
	}

	function &_sha1_step(&$H, $str) {
		$A = $H[0]; $B = $H[1]; $C = $H[2]; $D = $H[3]; $E = $H[4];
		$W = array_values(unpack('N16', $str));
		for ($i = 0; $i<16; ++$i) {
			$W[$i+16] = &$W[$i];
		}

		$t = 0;
		do {		//  0 <= t < 20
			$s = $t & 0xf;
			if ($t>=16) {
				$W[$s] = _sha1_s($W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s]);
			}

			$TEMP = ($D ^ ($B & ($C ^ $D))) + 0x5A827999 +
				 _sha1_s($A, 5) + $E + $W[$s];
			$E = $D; $D = $C; $C = _sha1_s($B, 30); $B = $A; $A = $TEMP;
		} while (++$t<20);

		do {		// 20 <= t < 40
			$s = $t & 0xf;
			$W[$s] = _sha1_s($W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s]);

			$TEMP = ($B ^ $C ^ $D) + 0x6ED9EBA1 +
				 _sha1_s($A, 5) + $E + $W[$s];
			$E = $D; $D = $C; $C = _sha1_s($B, 30); $B = $A; $A = $TEMP;
		} while (++$t<40);

		do {		// 40 <= t < 60
			$s = $t & 0xf;
			$W[$s] = _sha1_s($W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s]);

			$TEMP = (($B & $C) | ($D & ($B | $C))) + 0x8F1BBCDC +
				 _sha1_s($A, 5) + $E + $W[$s];
			$E = $D; $D = $C; $C = _sha1_s($B, 30); $B = $A; $A = $TEMP;
		} while (++$t<60);

		do {		// 60 <= t < 80
			$s = $t & 0xf;
			$W[$s] = _sha1_s($W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s]);

			$TEMP = ($B ^ $C ^ $D) + 0xCA62C1D6 +
				 _sha1_s($A, 5) + $E + $W[$s];
			$E = $D; $D = $C; $C = _sha1_s($B, 30); $B = $A; $A = $TEMP;
		} while (++$t<80);

		$H = array($H[0] + $A, $H[1] + $B, $H[2] + $C, $H[3] + $D, $H[4] + $E);
	}


	function &sha1_raw($str) {
		$l = strlen($str);
		$str = str_pad("$str\x80\0\0\0\0", ($l&~63)+((($l&63)<56)?60:124),
					   "\0") .  pack('N', $l<<3);
		$l = strlen($str);

		$H = array(0x67452301, 0xEFCDAB89, 0x98BADCFE, 0x10325476, 0xC3D2E1F0);
		for ($i = 0; $i<$l; $i += 64) {
			_sha1_step($H, substr($str, $i, 64));
		}

		return pack('N*', $H[0], $H[1], $H[2], $H[3], $H[4]);
	}

	function &sha1($str) {
		return bin2hex(sha1_raw($str));
	}
}

// }}}
// {{{ sha1_base64()

function &sha1_base64($str, $pad = false) {
	return base64_encode(sha1_raw($str)) . ($pad?'=':'');
}

// }}}
// {{{ Tests

$sha1 = "(using $sha1 to calculate SHA-1)";
if (sha1('abc')!=='a9993e364706816aba3e25717850c26c9cd0d89d') {
	trigger_error("SHA-1 Test #1 faild $sha1.", E_USER_ERROR);
}
if (sha1('abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq')!==
	'84983e441c3bd26ebaae4aa1f95129e5e54670f1') {
	trigger_error('SHA-1 Test #2 faild $sha1.', E_USER_ERROR);
}
/* Disabled becouse of time and memory consumption
if (sha1(str_pad('', 1e6, 'a'))!=='34aa973cd4c4daa4f61eeb2bdbad27316534016f') {
	trigger_error('SHA-1 Test #3 faild.', E_USER_ERROR);
}
// */
unset($sha1);

// }}}
?>